package com.github.davidmoten.guavamini;

import org.junit.Test;

import com.github.davidmoten.junit.Asserts;

public class Collections2Test {

    @Test
    public void isUtilityClass() {
        Asserts.assertIsUtilityClass(Collections2.class);
    }
}
