package com.github.davidmoten.guavamini.annotations;

/**
 * Annotates a program element that exists, or is more widely visible than
 * otherwise necessary, only for use in test code.
 */
public @interface VisibleForTesting {
}